@@ -211,6 +211,9 @@ class RouterSharedDriver(router_driver.RouterBaseDriver):
 
     def _update_nat_rules_on_routers(self, context,
                                      target_router_id, router_ids):
+        edge_id, az_name = self.plugin._get_edge_id_and_az_by_rtr_id(
+            context, target_router_id)
+        az = self._availability_zones.get_availability_zone(az_name)
         snats = []
         dnats = []
         vnics_by_router = self._get_all_routers_vnic_indices(
@@ -222,7 +225,7 @@ class RouterSharedDriver(router_driver.RouterBaseDriver):
                 snat, dnat = self.plugin._get_nat_rules(context, router)
                 snats.extend(snat)
                 dnats.extend(dnat)
-                if (not cfg.CONF.nsxv.bind_floatingip_to_all_interfaces and
+                if (not az.bind_floatingip_to_all_interfaces and
                     len(dnat) > 0):
                     # Copy each DNAT rule to all vnics of the other routers,
                     # to allow NAT-ed traffic between routers
@@ -241,7 +244,7 @@ class RouterSharedDriver(router_driver.RouterBaseDriver):
                             dnats.extend([new_rule])
 
         edge_utils.update_nat_rules(
-            self.nsx_v, context, target_router_id, snats, dnats)
+            self.nsx_v, context, target_router_id, snats, dnats, az=az)
 
     def _update_external_interface_on_routers(self, context,
                                               target_router_id, router_ids):
@@ -382,6 +385,8 @@ class RouterSharedDriver(router_driver.RouterBaseDriver):
         The router with static routes will be conflict with all other routers.
         The routers with different gateway will be conflict.
         The routers with overlapping interface will be conflict.
+        In not share_edges_between_tenants: The routers of different tenants
+            will be in conflict with the router
         """
         # 1. Check gateway
         # 2. Check subnet interface
@@ -398,6 +403,7 @@ class RouterSharedDriver(router_driver.RouterBaseDriver):
             router_dict = {}
             router_dict['id'] = r['id']
             router_dict['gateway'] = None
+            router_dict['tenant_id'] = r['tenant_id']
             for gwp in gw_ports:
                 if gwp['id'] == r['gw_port_id']:
                     try:
@@ -453,7 +459,12 @@ class RouterSharedDriver(router_driver.RouterBaseDriver):
                     if (conflict_ip_set & ip_set):
                         conflict_routers.append(r['id'])
                     else:
-                        available_routers.append(r['id'])
+                        if (not cfg.CONF.nsxv.share_edges_between_tenants and
+                            src_router_dict['tenant_id'] != r['tenant_id']):
+                            # routers of other tenants are conflicting
+                            conflict_routers.append(r['id'])
+                        else:
+                            available_routers.append(r['id'])
                 else:
                     conflict_routers.append(r['id'])
 
