@@ -20,6 +20,7 @@ import eventlet
 import netaddr
 from neutron_lib import constants
 from neutron_lib import context as neutron_context
+from neutron_lib.plugins import directory
 from oslo_config import cfg
 from oslo_log import log as logging
 
@@ -29,6 +30,7 @@ from vmware_nsx.common import locking
 from vmware_nsx.common import nsxv_constants
 from vmware_nsx.common import utils
 from vmware_nsx.db import nsxv_db
+from vmware_nsx.extensions import projectpluginmap
 from vmware_nsx.plugins.nsx_v.vshield import (
     nsxv_loadbalancer as nsxv_lb)
 from vmware_nsx.plugins.nsx_v.vshield.common import (
@@ -117,6 +119,19 @@ class NsxVMetadataProxyHandler(object):
 
         # Init cannot run concurrently on multiple nodes
         with locking.LockManager.get_lock('nsx-metadata-init'):
+            # if the core plugin is the TVD - we need to add project
+            # plugin mapping for the internal project
+            core_plugin = directory.get_plugin()
+            if core_plugin.is_tvd_plugin():
+                try:
+                    core_plugin.create_project_plugin_map(
+                        context,
+                        {'project_plugin_map':
+                            {'plugin': projectpluginmap.NsxPlugins.NSX_V,
+                             'project': nsxv_constants.INTERNAL_TENANT_ID}},
+                        internal=True)
+                except projectpluginmap.ProjectPluginAlreadyExists:
+                    pass
             self.internal_net, self.internal_subnet = (
                 self._get_internal_network_and_subnet(context))
 
@@ -165,7 +180,6 @@ class NsxVMetadataProxyHandler(object):
             return int_net['network_id']
 
     def _get_internal_network_and_subnet(self, context):
-
         # Try to find internal net, internal subnet. If not found, create new
         internal_net = self._get_internal_net_by_az(context)
         internal_subnet = None
@@ -194,9 +208,10 @@ class NsxVMetadataProxyHandler(object):
                         self.nsxv_plugin.delete_network(context,
                                                         internal_net)
 
-                    LOG.exception("Exception %s while creating internal "
-                                  "network for metadata service", e)
-                    return
+                    error = (_("Exception %s while creating internal "
+                               "network for metadata service") % e)
+                    LOG.exception(error)
+                    raise nsxv_exc.NsxPluginException(err_msg=error)
 
                 # Update the new network_id in DB
                 nsxv_db.create_nsxv_internal_network(
