@@ -87,6 +87,8 @@ NETWORK_TYPES = ['Network', 'VirtualWire', 'DistributedVirtualPortgroup']
 # Dynamic routing constants
 ROUTING_CONFIG = "routing/config"
 BGP_ROUTING_CONFIG = "routing/config/bgp"
+ELAPSED_TIME_THRESHOLD = 30
+MAX_EDGE_DEPLOY_TIMEOUT = 1200
 
 
 def retry_upon_exception_exclude_error_codes(
@@ -111,23 +113,21 @@ class Vcns(object):
         self.password = password
         self.ca_file = ca_file
         self.insecure = insecure
-        self.jsonapi_client = VcnsApiClient.VcnsApiHelper(address, user,
-                                                          password,
-                                                          format='json',
-                                                          ca_file=ca_file,
-                                                          insecure=insecure)
-        self.xmlapi_client = VcnsApiClient.VcnsApiHelper(address, user,
-                                                         password,
-                                                         format='xml',
-                                                         ca_file=ca_file,
-                                                         insecure=insecure)
+        self.jsonapi_client = VcnsApiClient.VcnsApiHelper(
+            address, user, password, format='json', ca_file=ca_file,
+            insecure=insecure, timeout=cfg.CONF.nsxv.nsx_transaction_timeout)
+        self.xmlapi_client = VcnsApiClient.VcnsApiHelper(
+            address, user, password, format='xml', ca_file=ca_file,
+            insecure=insecure, timeout=cfg.CONF.nsxv.nsx_transaction_timeout)
         self._nsx_version = None
         self._normalized_scoping_objects = None
+        self._normalized_global_objects = None
 
     @retry_upon_exception(exceptions.ServiceConflict)
     def _client_request(self, client, method, uri,
-                        params, headers, encodeParams):
-        return client(method, uri, params, headers, encodeParams)
+                        params, headers, encodeParams, timeout=None):
+        return client(method, uri, params, headers, encodeParams,
+                      timeout=timeout)
 
     def do_request(self, method, uri, params=None, format='json', **kwargs):
         msg = ("VcnsApiHelper('%(method)s', '%(uri)s', '%(body)s')" %
@@ -143,14 +143,24 @@ class Vcns(object):
         else:
             _client = self.xmlapi_client.request
 
+        timeout = kwargs.get('timeout')
         ts = time.time()
         header, content = self._client_request(_client, method, uri, params,
-                                               headers, encodeParams)
+                                               headers, encodeParams,
+                                               timeout=timeout)
         te = time.time()
+        elapsed_time = te - ts
+
+        LOG.debug('VcnsApiHelper for %(method)s %(uri)s took %(seconds)2.4f. '
+                  'reply: header=%(header)s content=%(content)s',
+                  {'method': method, 'uri': uri,
+                   'header': header, 'content': content,
+                   'seconds': elapsed_time})
+        if elapsed_time > ELAPSED_TIME_THRESHOLD:
+            LOG.warning('Vcns call for %(method)s %(uri)s took %(seconds)2.4f',
+                        {'method': method, 'uri': uri,
+                         'seconds': elapsed_time})
 
-        LOG.debug('VcnsApiHelper reply: header=%(header)s content=%(content)s'
-                  ' took %(seconds)2.4f',
-                  {'header': header, 'content': content, 'seconds': te - ts})
         if content == '':
             return header, {}
         if kwargs.get('decode', True):
@@ -165,7 +175,8 @@ class Vcns(object):
     @retry_upon_exception(exceptions.RequestBad)
     def deploy_edge(self, request):
         uri = URI_PREFIX
-        return self.do_request(HTTP_POST, uri, request, decode=False)
+        return self.do_request(HTTP_POST, uri, request, decode=False,
+                               timeout=MAX_EDGE_DEPLOY_TIMEOUT)
 
     def update_edge(self, edge_id, request):
         uri = "%s/%s" % (URI_PREFIX, edge_id)
@@ -815,9 +826,17 @@ class Vcns(object):
                   'approvedMacAddress': mac_addr,
                   'publishedIpAddress': addresses,
                   'publishedMacAddress': mac_addr}}}
-
-        return self.do_request(HTTP_POST, '%s?action=approve' % uri,
-                               body, format='xml', decode=False)
+        try:
+            return self.do_request(HTTP_POST, '%s?action=approve' % uri,
+                                   body, format='xml', decode=False)
+        except exceptions.VcnsApiException as e:
+            nsx_errcode = self.xmlapi_client._get_nsx_errorcode(e.response)
+            if nsx_errcode == constants.NSX_ERROR_ALREADY_EXISTS:
+                LOG.warning("Spoofguard entry for %s already exists",
+                            vnic_id)
+                raise exceptions.AlreadyExists(resource=vnic_id)
+            # raise original exception for retries
+            raise
 
     @retry_upon_exception(exceptions.RequestBad)
     def approve_assigned_addresses(self, policy_id,
@@ -1145,3 +1164,32 @@ class Vcns(object):
     def delete_bgp_routing_config(self, edge_id):
         uri = self._build_uri_path(edge_id, BGP_ROUTING_CONFIG)
         return self.do_request(HTTP_DELETE, uri)
+
+    def get_global_objects(self):
+        uri = '%s/application/scope/globalroot-0' % SERVICES_PREFIX
+        h, scoping_objects = self.do_request(HTTP_GET, uri, decode=False,
+                                             format='xml')
+        return scoping_objects
+
+    def _globalobjects_lookup(self, name, use_cache=False):
+        """Return objectId a specific name in the NSX global objects."""
+        # used cached scoping objects during plugin init since it is
+        # a big structure to retrieve and parse each time.
+        if use_cache and self._normalized_global_objects is not None:
+            # Use the cached data
+            root = self._normalized_global_objects
+        else:
+            # Not using cache, or we do want to use it,
+            # but it was not saved yet:
+            # So get the data from the NSX and parse it
+            so_list = self.get_global_objects()
+            root = utils.normalize_xml(so_list)
+            # Save it for possible usage next time (even if not using cache)
+            self._normalized_global_objects = root
+
+        for obj in root.iter('application'):
+            if obj.find('name').text == name:
+                return obj.find('objectId').text
+
+    def get_application_id(self, name):
+        return self._globalobjects_lookup(name, use_cache=True)
