@@ -31,6 +31,7 @@ from vmware_nsx.common import exceptions as nsx_exc
 from vmware_nsx.common import utils
 from vmware_nsx.db import db as nsx_db
 from vmware_nsx.extensions import advancedserviceproviders as as_providers
+from vmware_nsx.plugins.nsx_v3 import availability_zones as nsx_az
 from vmware_nsx.tests.unit.nsx_v3 import test_plugin
 from vmware_nsxlib.v3 import nsx_constants
 from vmware_nsxlib.v3 import resources as nsx_resources
@@ -287,13 +288,19 @@ class NsxNativeDhcpTestCase(test_plugin.NsxV3PluginTestCaseMixin):
 
     def test_dhcp_service_with_create_dhcp_subnet_in_vlan_network(self):
         # Test if a DHCP-enabled subnet cannot be created in a vlan network.
+        # on nsx version that does not support it
         povidernet_args = {pnet.NETWORK_TYPE: 'vlan',
                            pnet.PHYSICAL_NETWORK: 'tzuuid',
                            pnet.SEGMENTATION_ID: 100}
-        with self.network(providernet_args=povidernet_args,
-                          arg_list=(pnet.NETWORK_TYPE,
-                                    pnet.PHYSICAL_NETWORK,
-                                    pnet.SEGMENTATION_ID)) as network:
+        with mock.patch(
+            'vmware_nsxlib.v3.core_resources.NsxLibTransportZone.'
+            'get_transport_type', return_value='VLAN'),\
+            mock.patch.object(self.plugin.nsxlib, 'feature_supported',
+                              return_value=False),\
+            self.network(providernet_args=povidernet_args,
+                         arg_list=(pnet.NETWORK_TYPE,
+                                   pnet.PHYSICAL_NETWORK,
+                                   pnet.SEGMENTATION_ID)) as network:
             subnet = {'subnet': {'network_id': network['network']['id'],
                                  'cidr': '10.0.0.0/24',
                                  'enable_dhcp': True}}
@@ -843,7 +850,7 @@ class NsxNativeDhcpTestCase(test_plugin.NsxV3PluginTestCaseMixin):
         net = p.create_network(ctx, data)
         self.assertEqual([],
                          net['availability_zone_hints'])
-        self.assertEqual(['default'],
+        self.assertEqual([nsx_az.DEFAULT_NAME],
                          net['availability_zones'])
 
     def test_dhcp_service_with_create_az_network(self):
