@@ -15,16 +15,9 @@
 
 import mock
 import netaddr
-import six
-from webob import exc
-
-from neutron.api.v2 import attributes
 from neutron.db import models_v2
 from neutron.extensions import address_scope
-from neutron.extensions import external_net
-from neutron.extensions import extraroute
 from neutron.extensions import l3
-from neutron.extensions import l3_ext_gw_mode
 from neutron.extensions import securitygroup as secgrp
 from neutron.tests.unit import _test_extension_portbindings as test_bindings
 from neutron.tests.unit.db import test_db_base_plugin_v2 as test_plugin
@@ -36,10 +29,13 @@ from neutron.tests.unit.extensions \
     import test_l3_ext_gw_mode as test_ext_gw_mode
 from neutron.tests.unit.scheduler \
     import test_dhcp_agent_scheduler as test_dhcpagent
-
-from neutron_lib.api.definitions import address_scope as addr_apidef
+from neutron_lib.api.definitions import external_net as extnet_apidef
+from neutron_lib.api.definitions import extraroute as xroute_apidef
+from neutron_lib.api.definitions import l3_ext_gw_mode as l3_egm_apidef
+from neutron_lib.api.definitions import port_security as psec
 from neutron_lib.api.definitions import portbindings
 from neutron_lib.api.definitions import provider_net as pnet
+from neutron_lib.api.definitions import vlantransparent as vlan_apidef
 from neutron_lib.callbacks import exceptions as nc_exc
 from neutron_lib import constants
 from neutron_lib import context
@@ -47,6 +43,7 @@ from neutron_lib import exceptions as n_exc
 from neutron_lib.plugins import directory
 from oslo_config import cfg
 from oslo_utils import uuidutils
+from webob import exc
 
 from vmware_nsx.api_client import exception as api_exc
 from vmware_nsx.common import utils
@@ -64,6 +61,7 @@ NSX_TZ_NAME = 'default transport zone'
 NSX_DHCP_PROFILE_ID = 'default dhcp profile'
 NSX_METADATA_PROXY_ID = 'default metadata proxy'
 NSX_SWITCH_PROFILE = 'dummy switch profile'
+NSX_DHCP_RELAY_SRV = 'dhcp relay srv'
 
 
 def _mock_create_firewall_rules(*args):
@@ -77,6 +75,10 @@ def _mock_create_firewall_rules(*args):
         ]}
 
 
+def _return_id_key(*args, **kwargs):
+    return {'id': uuidutils.generate_uuid()}
+
+
 def _mock_nsx_backend_calls():
     mock.patch("vmware_nsxlib.v3.client.NSX3Client").start()
 
@@ -84,9 +86,6 @@ def _mock_nsx_backend_calls():
                     'resource_type': 'FakeResource',
                     'id': uuidutils.generate_uuid()}
 
-    def _return_id_key(*args, **kwargs):
-        return {'id': uuidutils.generate_uuid()}
-
     def _return_id(*args, **kwargs):
         return uuidutils.generate_uuid()
 
@@ -112,11 +111,6 @@ def _mock_nsx_backend_calls():
         "get_id_by_name_or_id",
         return_value=uuidutils.generate_uuid()).start()
 
-    mock.patch(
-        "vmware_nsxlib.v3.core_resources.NsxLibTransportZone."
-        "get_id_by_name_or_id",
-        return_value=uuidutils.generate_uuid()).start()
-
     mock.patch(
         "vmware_nsxlib.v3.core_resources.NsxLibBridgeEndpoint.create",
         side_effect=_return_id_key).start()
@@ -134,6 +128,11 @@ def _mock_nsx_backend_calls():
         "get_id_by_name_or_id",
         return_value=NSX_DHCP_PROFILE_ID).start()
 
+    mock.patch(
+        "vmware_nsxlib.v3.core_resources.NsxLibDhcpRelayService."
+        "get_id_by_name_or_id",
+        return_value=NSX_DHCP_RELAY_SRV).start()
+
     mock.patch(
         "vmware_nsxlib.v3.core_resources.NsxLibMetadataProxy."
         "get_id_by_name_or_id",
@@ -162,7 +161,7 @@ def _mock_nsx_backend_calls():
 
     mock.patch(
         "vmware_nsxlib.v3.NsxLib.get_version",
-        return_value='1.1.0').start()
+        return_value='2.2.0').start()
 
     mock.patch(
         "vmware_nsxlib.v3.load_balancer.Service.get_router_lb_service",
@@ -188,12 +187,6 @@ class NsxV3PluginTestCaseMixin(test_plugin.NeutronDbPluginV2TestCase,
         mock_process_security_group_logging = mock.patch.object(
             nsx_plugin.NsxV3Plugin, '_process_security_group_logging')
         mock_process_security_group_logging.start()
-        # need to mock the global placeholder. This is due to the fact that
-        # the generic security group tests assume that there is just one
-        # security group.
-        mock_ensure_global_sg_placeholder = mock.patch.object(
-            nsx_plugin.NsxV3Plugin, '_ensure_global_sg_placeholder')
-        mock_ensure_global_sg_placeholder.start()
 
     def setUp(self, plugin=PLUGIN_NAME,
               ext_mgr=None,
@@ -204,6 +197,9 @@ class NsxV3PluginTestCaseMixin(test_plugin.NeutronDbPluginV2TestCase,
         _mock_nsx_backend_calls()
         self.setup_conf_overrides()
         self.mock_plugin_methods()
+        # ignoring the given plugin and use the nsx-v3 one
+        if not plugin.endswith('NsxTVDPlugin'):
+            plugin = PLUGIN_NAME
         super(NsxV3PluginTestCaseMixin, self).setUp(plugin=plugin,
                                                     ext_mgr=ext_mgr)
 
@@ -226,8 +222,8 @@ class NsxV3PluginTestCaseMixin(test_plugin.NeutronDbPluginV2TestCase,
         # attributes containing a colon to be passed with
         # a double underscore instead
         kwargs = dict((k.replace('__', ':'), v) for k, v in kwargs.items())
-        if external_net.EXTERNAL in kwargs:
-            arg_list = (external_net.EXTERNAL, ) + (arg_list or ())
+        if extnet_apidef.EXTERNAL in kwargs:
+            arg_list = (extnet_apidef.EXTERNAL, ) + (arg_list or ())
 
         if providernet_args:
             kwargs.update(providernet_args)
@@ -261,9 +257,32 @@ class NsxV3PluginTestCaseMixin(test_plugin.NeutronDbPluginV2TestCase,
             with ctx.session.begin(subtransactions=True):
                 ctx.session.add(models_v2.Network(id=network_id))
 
+    def _enable_dhcp_relay(self):
+        # Add the relay service to the config and availability zones
+        cfg.CONF.set_override('native_dhcp_metadata', True, 'nsx_v3')
+        cfg.CONF.set_override('dhcp_relay_service', NSX_DHCP_RELAY_SRV,
+                              'nsx_v3')
+        mock_nsx_version = mock.patch.object(
+            self.plugin.nsxlib, 'feature_supported', return_value=True)
+        mock_nsx_version.start()
+        self.plugin.init_availability_zones()
+        for az in self.plugin.get_azs_list():
+            az.translate_configured_names_to_uuids(self.plugin.nsxlib)
+
 
 class TestNetworksV2(test_plugin.TestNetworksV2, NsxV3PluginTestCaseMixin):
 
+    def setUp(self, plugin=PLUGIN_NAME,
+              ext_mgr=None,
+              service_plugins=None):
+        # add vlan transparent to the configuration
+        cfg.CONF.set_override('vlan_transparent', True)
+        super(TestNetworksV2, self).setUp(plugin=plugin,
+                                          ext_mgr=ext_mgr)
+
+    def tearDown(self):
+        super(TestNetworksV2, self).tearDown()
+
     @mock.patch.object(nsx_plugin.NsxV3Plugin, 'validate_availability_zones')
     def test_create_network_with_availability_zone(self, mock_validate_az):
         name = 'net-with-zone'
@@ -284,6 +303,249 @@ class TestNetworksV2(test_plugin.TestNetworksV2, NsxV3PluginTestCaseMixin):
             networks = self.plugin.get_networks(ctx)
             self.assertListEqual([], networks)
 
+    def test_create_provider_flat_network(self):
+        providernet_args = {pnet.NETWORK_TYPE: 'flat'}
+        with mock.patch('vmware_nsxlib.v3.core_resources.NsxLibLogicalSwitch.'
+                        'create', side_effect=_return_id_key) as nsx_create, \
+            mock.patch('vmware_nsxlib.v3.core_resources.NsxLibLogicalSwitch.'
+                       'delete') as nsx_delete, \
+            mock.patch('vmware_nsxlib.v3.core_resources.NsxLibTransportZone.'
+                       'get_transport_type', return_value='VLAN'),\
+            self.network(name='flat_net',
+                         providernet_args=providernet_args,
+                         arg_list=(pnet.NETWORK_TYPE, )) as net:
+            self.assertEqual('flat', net['network'].get(pnet.NETWORK_TYPE))
+            # make sure the network is created at the backend
+            nsx_create.assert_called_once()
+
+            # Delete the network and make sure it is deleted from the backend
+            req = self.new_delete_request('networks', net['network']['id'])
+            res = req.get_response(self.api)
+            self.assertEqual(exc.HTTPNoContent.code, res.status_int)
+            nsx_delete.assert_called_once()
+
+    def test_create_provider_flat_network_with_physical_net(self):
+        physical_network = nsx_v3_mocks.DEFAULT_TIER0_ROUTER_UUID
+        providernet_args = {pnet.NETWORK_TYPE: 'flat',
+                            pnet.PHYSICAL_NETWORK: physical_network}
+        with mock.patch(
+            'vmware_nsxlib.v3.core_resources.NsxLibTransportZone.'
+            'get_transport_type', return_value='VLAN'),\
+            self.network(name='flat_net',
+                         providernet_args=providernet_args,
+                         arg_list=(pnet.NETWORK_TYPE,
+                                   pnet.PHYSICAL_NETWORK)) as net:
+            self.assertEqual('flat', net['network'].get(pnet.NETWORK_TYPE))
+
+    def test_create_provider_flat_network_with_vlan(self):
+        providernet_args = {pnet.NETWORK_TYPE: 'flat',
+                            pnet.SEGMENTATION_ID: 11}
+        with mock.patch('vmware_nsxlib.v3.core_resources.NsxLibTransportZone.'
+                        'get_transport_type', return_value='VLAN'):
+            result = self._create_network(fmt='json', name='bad_flat_net',
+                                          admin_state_up=True,
+                                          providernet_args=providernet_args,
+                                          arg_list=(pnet.NETWORK_TYPE,
+                                                    pnet.SEGMENTATION_ID))
+            data = self.deserialize('json', result)
+            # should fail
+            self.assertEqual('InvalidInput', data['NeutronError']['type'])
+
+    def test_create_provider_geneve_network(self):
+        providernet_args = {pnet.NETWORK_TYPE: 'geneve'}
+        with mock.patch('vmware_nsxlib.v3.core_resources.NsxLibLogicalSwitch.'
+                        'create', side_effect=_return_id_key) as nsx_create, \
+            mock.patch('vmware_nsxlib.v3.core_resources.NsxLibLogicalSwitch.'
+                       'delete') as nsx_delete, \
+            mock.patch('vmware_nsxlib.v3.core_resources.NsxLibTransportZone.'
+                       'get_transport_type', return_value='OVERLAY'),\
+            self.network(name='geneve_net',
+                         providernet_args=providernet_args,
+                         arg_list=(pnet.NETWORK_TYPE, )) as net:
+            self.assertEqual('geneve', net['network'].get(pnet.NETWORK_TYPE))
+            # make sure the network is created at the backend
+            nsx_create.assert_called_once()
+
+            # Delete the network and make sure it is deleted from the backend
+            req = self.new_delete_request('networks', net['network']['id'])
+            res = req.get_response(self.api)
+            self.assertEqual(exc.HTTPNoContent.code, res.status_int)
+            nsx_delete.assert_called_once()
+
+    def test_create_provider_geneve_network_with_physical_net(self):
+        physical_network = nsx_v3_mocks.DEFAULT_TIER0_ROUTER_UUID
+        providernet_args = {pnet.NETWORK_TYPE: 'geneve',
+                            pnet.PHYSICAL_NETWORK: physical_network}
+        with mock.patch(
+            'vmware_nsxlib.v3.core_resources.NsxLibTransportZone.'
+            'get_transport_type', return_value='OVERLAY'),\
+            self.network(name='geneve_net',
+                         providernet_args=providernet_args,
+                         arg_list=(pnet.NETWORK_TYPE, )) as net:
+            self.assertEqual('geneve', net['network'].get(pnet.NETWORK_TYPE))
+
+    def test_create_provider_geneve_network_with_vlan(self):
+        providernet_args = {pnet.NETWORK_TYPE: 'geneve',
+                            pnet.SEGMENTATION_ID: 11}
+        with mock.patch(
+            'vmware_nsxlib.v3.core_resources.NsxLibTransportZone.'
+            'get_transport_type', return_value='OVERLAY'):
+            result = self._create_network(fmt='json', name='bad_geneve_net',
+                                          admin_state_up=True,
+                                          providernet_args=providernet_args,
+                                          arg_list=(pnet.NETWORK_TYPE,
+                                                    pnet.SEGMENTATION_ID))
+            data = self.deserialize('json', result)
+            # should fail
+            self.assertEqual('InvalidInput', data['NeutronError']['type'])
+
+    def test_create_provider_vlan_network(self):
+        providernet_args = {pnet.NETWORK_TYPE: 'vlan',
+                            pnet.SEGMENTATION_ID: 11}
+        with mock.patch('vmware_nsxlib.v3.core_resources.NsxLibLogicalSwitch.'
+                        'create', side_effect=_return_id_key) as nsx_create, \
+            mock.patch('vmware_nsxlib.v3.core_resources.NsxLibLogicalSwitch.'
+                       'delete') as nsx_delete, \
+            mock.patch('vmware_nsxlib.v3.core_resources.NsxLibTransportZone.'
+                       'get_transport_type', return_value='VLAN'),\
+            self.network(name='vlan_net',
+                         providernet_args=providernet_args,
+                         arg_list=(pnet.NETWORK_TYPE,
+                                   pnet.SEGMENTATION_ID)) as net:
+            self.assertEqual('vlan', net['network'].get(pnet.NETWORK_TYPE))
+            # make sure the network is created at the backend
+            nsx_create.assert_called_once()
+
+            # Delete the network and make sure it is deleted from the backend
+            req = self.new_delete_request('networks', net['network']['id'])
+            res = req.get_response(self.api)
+            self.assertEqual(exc.HTTPNoContent.code, res.status_int)
+            nsx_delete.assert_called_once()
+
+    def test_create_provider_nsx_network(self):
+        physical_network = 'Fake logical switch'
+        providernet_args = {pnet.NETWORK_TYPE: 'nsx-net',
+                            pnet.PHYSICAL_NETWORK: physical_network}
+
+        with mock.patch(
+            'vmware_nsxlib.v3.core_resources.NsxLibLogicalSwitch.create',
+            side_effect=nsxlib_exc.ResourceNotFound) as nsx_create, \
+            mock.patch('vmware_nsxlib.v3.core_resources.NsxLibLogicalSwitch.'
+                       'delete') as nsx_delete, \
+            self.network(name='nsx_net',
+                         providernet_args=providernet_args,
+                         arg_list=(pnet.NETWORK_TYPE,
+                                   pnet.PHYSICAL_NETWORK)) as net:
+            self.assertEqual('nsx-net', net['network'].get(pnet.NETWORK_TYPE))
+            self.assertEqual(physical_network,
+                             net['network'].get(pnet.PHYSICAL_NETWORK))
+            # make sure the network is NOT created at the backend
+            nsx_create.assert_not_called()
+
+            # Delete the network. It should NOT deleted from the backend
+            req = self.new_delete_request('networks', net['network']['id'])
+            res = req.get_response(self.api)
+            self.assertEqual(exc.HTTPNoContent.code, res.status_int)
+            nsx_delete.assert_not_called()
+
+    def test_create_provider_bad_nsx_network(self):
+        physical_network = 'Bad logical switch'
+        providernet_args = {pnet.NETWORK_TYPE: 'nsx-net',
+                            pnet.PHYSICAL_NETWORK: physical_network}
+        with mock.patch(
+            "vmware_nsxlib.v3.core_resources.NsxLibLogicalSwitch.get",
+            side_effect=nsxlib_exc.ResourceNotFound):
+            result = self._create_network(fmt='json', name='bad_nsx_net',
+                                          admin_state_up=True,
+                                          providernet_args=providernet_args,
+                                          arg_list=(pnet.NETWORK_TYPE,
+                                                    pnet.PHYSICAL_NETWORK))
+            data = self.deserialize('json', result)
+            # should fail
+            self.assertEqual('InvalidInput', data['NeutronError']['type'])
+
+    def test_create_ens_network_with_no_port_sec(self):
+        cfg.CONF.set_override('ens_support', True, 'nsx_v3')
+        providernet_args = {psec.PORTSECURITY: False}
+        with mock.patch("vmware_nsxlib.v3.core_resources.NsxLibTransportZone."
+                        "get_host_switch_mode", return_value="ENS"),\
+            mock.patch(
+            "vmware_nsxlib.v3.core_resources.NsxLibLogicalSwitch.get",
+            return_value={'transport_zone_id': 'xxx'}):
+
+            result = self._create_network(fmt='json', name='ens_net',
+                                          admin_state_up=True,
+                                          providernet_args=providernet_args,
+                                          arg_list=(psec.PORTSECURITY,))
+            res = self.deserialize('json', result)
+            # should succeed, and net should have port security disabled
+            self.assertFalse(res['network']['port_security_enabled'])
+
+    def test_create_ens_network_with_port_sec(self):
+        cfg.CONF.set_override('ens_support', True, 'nsx_v3')
+        providernet_args = {psec.PORTSECURITY: True}
+        with mock.patch("vmware_nsxlib.v3.core_resources.NsxLibTransportZone."
+                        "get_host_switch_mode", return_value="ENS"),\
+            mock.patch(
+            "vmware_nsxlib.v3.core_resources.NsxLibLogicalSwitch.get",
+            return_value={'transport_zone_id': 'xxx'}):
+            result = self._create_network(fmt='json', name='ens_net',
+                                          admin_state_up=True,
+                                          providernet_args=providernet_args,
+                                          arg_list=(psec.PORTSECURITY,))
+            res = self.deserialize('json', result)
+            # should fail
+            self.assertEqual('NsxENSPortSecurity',
+                             res['NeutronError']['type'])
+
+    def test_update_ens_network(self):
+        cfg.CONF.set_override('ens_support', True, 'nsx_v3')
+        providernet_args = {psec.PORTSECURITY: False}
+        with mock.patch("vmware_nsxlib.v3.core_resources.NsxLibTransportZone."
+                        "get_host_switch_mode", return_value="ENS"),\
+            mock.patch(
+            "vmware_nsxlib.v3.core_resources.NsxLibLogicalSwitch.get",
+            return_value={'transport_zone_id': 'xxx'}):
+
+            result = self._create_network(fmt='json', name='ens_net',
+                                          admin_state_up=True,
+                                          providernet_args=providernet_args,
+                                          arg_list=(psec.PORTSECURITY,))
+            net = self.deserialize('json', result)
+            net_id = net['network']['id']
+            args = {'network': {psec.PORTSECURITY: True}}
+            req = self.new_update_request('networks', args,
+                                          net_id, fmt='json')
+            res = self.deserialize('json', req.get_response(self.api))
+            # should fail
+            self.assertEqual('NsxENSPortSecurity',
+                             res['NeutronError']['type'])
+
+    def test_create_transparent_vlan_network(self):
+        providernet_args = {vlan_apidef.VLANTRANSPARENT: True}
+        with mock.patch(
+            'vmware_nsxlib.v3.core_resources.NsxLibTransportZone.'
+            'get_transport_type', return_value='OVERLAY'),\
+            self.network(name='vt_net',
+                         providernet_args=providernet_args,
+                         arg_list=(vlan_apidef.VLANTRANSPARENT, )) as net:
+            self.assertTrue(net['network'].get(vlan_apidef.VLANTRANSPARENT))
+
+    def test_create_provider_vlan_network_with_transparent(self):
+        providernet_args = {pnet.NETWORK_TYPE: 'vlan',
+                            vlan_apidef.VLANTRANSPARENT: True}
+        with mock.patch('vmware_nsxlib.v3.core_resources.NsxLibTransportZone.'
+                       'get_transport_type', return_value='VLAN'):
+            result = self._create_network(fmt='json', name='badvlan_net',
+                                          admin_state_up=True,
+                                          providernet_args=providernet_args,
+                                          arg_list=(
+                                              pnet.NETWORK_TYPE,
+                                              pnet.SEGMENTATION_ID,
+                                              vlan_apidef.VLANTRANSPARENT))
+            data = self.deserialize('json', result)
+            self.assertEqual('vlan', data['network'].get(pnet.NETWORK_TYPE))
+
 
 class TestSubnetsV2(test_plugin.TestSubnetsV2, NsxV3PluginTestCaseMixin):
 
@@ -301,6 +563,34 @@ class TestSubnetsV2(test_plugin.TestSubnetsV2, NsxV3PluginTestCaseMixin):
     def test_subnet_update_ipv4_and_ipv6_pd_slaac_subnets(self):
         self.skipTest('Multiple fixed ips on a port are not supported')
 
+    def test_subnet_native_dhcp_subnet_enabled(self):
+        cfg.CONF.set_override('native_dhcp_metadata', True, 'nsx_v3')
+        with self.network() as network:
+            with mock.patch.object(self.plugin,
+                                   '_enable_native_dhcp') as enable_dhcp,\
+                self.subnet(network=network, enable_dhcp=True):
+                # Native dhcp should be set for this subnet
+                self.assertTrue(enable_dhcp.called)
+
+    def test_subnet_native_dhcp_subnet_disabled(self):
+        cfg.CONF.set_override('native_dhcp_metadata', True, 'nsx_v3')
+        with self.network() as network:
+            with mock.patch.object(self.plugin,
+                                   '_enable_native_dhcp') as enable_dhcp,\
+                self.subnet(network=network, enable_dhcp=False):
+                # Native dhcp should be set for this subnet
+                self.assertFalse(enable_dhcp.called)
+
+    def test_subnet_native_dhcp_with_relay(self):
+        """Verify that the relay service is added to the router interface"""
+        self._enable_dhcp_relay()
+        with self.network() as network:
+            with mock.patch.object(self.plugin,
+                                  '_enable_native_dhcp') as enable_dhcp,\
+                self.subnet(network=network, enable_dhcp=True):
+                # Native dhcp should not be set for this subnet
+                self.assertFalse(enable_dhcp.called)
+
 
 class TestPortsV2(test_plugin.TestPortsV2, NsxV3PluginTestCaseMixin,
                   test_bindings.PortBindingsTestCase,
@@ -335,6 +625,18 @@ class TestPortsV2(test_plugin.TestPortsV2, NsxV3PluginTestCaseMixin,
                 self.assertEqual(res['port']['fixed_ips'],
                                  data['port']['fixed_ips'])
 
+    def test_delete_dhcp_port(self):
+        cfg.CONF.set_override('native_dhcp_metadata', True, 'nsx_v3')
+        with self.subnet():
+            pl = directory.get_plugin()
+            ctx = context.Context(user_id=None, tenant_id=self._tenant_id,
+                                  is_admin=False)
+            ports = pl.get_ports(
+                ctx, filters={'device_owner': [constants.DEVICE_OWNER_DHCP]})
+            req = self.new_delete_request('ports', ports[0]['id'])
+            res = req.get_response(self.api)
+            self.assertEqual(exc.HTTPBadRequest.code, res.status_int)
+
     def test_fail_create_port_with_ext_net(self):
         expected_error = 'InvalidInput'
         with self._create_l3_ext_network() as network:
@@ -590,6 +892,62 @@ class TestPortsV2(test_plugin.TestPortsV2, NsxV3PluginTestCaseMixin,
                 # configured one
                 self.assertEqual(expected_prof, actual_profs[0])
 
+    def test_create_ens_port_with_no_port_sec(self):
+        with self.subnet() as subnet,\
+            mock.patch("vmware_nsxlib.v3.core_resources.NsxLibTransportZone."
+                       "get_host_switch_mode", return_value="ENS"),\
+            mock.patch(
+            "vmware_nsxlib.v3.core_resources.NsxLibLogicalSwitch.get",
+            return_value={'transport_zone_id': 'xxx'}):
+            args = {'port': {'network_id': subnet['subnet']['network_id'],
+                             'tenant_id': subnet['subnet']['tenant_id'],
+                             'fixed_ips': [{'subnet_id':
+                                            subnet['subnet']['id']}],
+                             psec.PORTSECURITY: False}}
+            port_req = self.new_create_request('ports', args)
+            port = self.deserialize(self.fmt, port_req.get_response(self.api))
+            self.assertFalse(port['port']['port_security_enabled'])
+
+    def test_create_ens_port_with_port_sec(self):
+        with self.subnet() as subnet,\
+            mock.patch("vmware_nsxlib.v3.core_resources.NsxLibTransportZone."
+                       "get_host_switch_mode", return_value="ENS"),\
+            mock.patch(
+            "vmware_nsxlib.v3.core_resources.NsxLibLogicalSwitch.get",
+            return_value={'transport_zone_id': 'xxx'}):
+            args = {'port': {'network_id': subnet['subnet']['network_id'],
+                             'tenant_id': subnet['subnet']['tenant_id'],
+                             'fixed_ips': [{'subnet_id':
+                                            subnet['subnet']['id']}],
+                             psec.PORTSECURITY: True}}
+            port_req = self.new_create_request('ports', args)
+            res = self.deserialize('json', port_req.get_response(self.api))
+            # should fail
+            self.assertEqual('NsxENSPortSecurity',
+                             res['NeutronError']['type'])
+
+    def test_update_ens_port(self):
+        with self.subnet() as subnet,\
+            mock.patch("vmware_nsxlib.v3.core_resources.NsxLibTransportZone."
+                       "get_host_switch_mode", return_value="ENS"),\
+            mock.patch(
+            "vmware_nsxlib.v3.core_resources.NsxLibLogicalSwitch.get",
+            return_value={'transport_zone_id': 'xxx'}):
+            args = {'port': {'network_id': subnet['subnet']['network_id'],
+                             'tenant_id': subnet['subnet']['tenant_id'],
+                             'fixed_ips': [{'subnet_id':
+                                            subnet['subnet']['id']}],
+                             psec.PORTSECURITY: False}}
+            port_req = self.new_create_request('ports', args)
+            port = self.deserialize(self.fmt, port_req.get_response(self.api))
+            port_id = port['port']['id']
+            args = {'port': {psec.PORTSECURITY: True}}
+            req = self.new_update_request('ports', args, port_id)
+            res = self.deserialize('json', req.get_response(self.api))
+            # should fail
+            self.assertEqual('NsxENSPortSecurity',
+                             res['NeutronError']['type'])
+
     def test_update_port_update_ip_address_only(self):
         self.skipTest('Multiple fixed ips on a port are not supported')
 
@@ -614,6 +972,29 @@ class TestPortsV2(test_plugin.TestPortsV2, NsxV3PluginTestCaseMixin,
     def test_create_port_anticipating_allocation(self):
         self.skipTest('Multiple fixed ips on a port are not supported')
 
+    def test_create_compute_port_with_relay_no_router(self):
+        """Compute port creation should fail
+
+        if a network with dhcp relay is not connected to a router
+        """
+        self._enable_dhcp_relay()
+        with self.network() as network, \
+            self.subnet(network=network, enable_dhcp=True) as s1:
+            device_owner = constants.DEVICE_OWNER_COMPUTE_PREFIX + 'X'
+            data = {'port': {
+                        'network_id': network['network']['id'],
+                        'tenant_id': self._tenant_id,
+                        'name': 'port',
+                        'admin_state_up': True,
+                        'device_id': 'fake_device',
+                        'device_owner': device_owner,
+                        'fixed_ips': [{'subnet_id': s1['subnet']['id']}],
+                        'mac_address': '00:00:00:00:00:01'}
+                    }
+            self.assertRaises(n_exc.InvalidInput,
+                              self.plugin.create_port,
+                              self.ctx, data)
+
 
 class DHCPOptsTestCase(test_dhcpopts.TestExtraDhcpOpt,
                        NsxV3PluginTestCaseMixin):
@@ -642,17 +1023,10 @@ class TestL3ExtensionManager(object):
 
     def get_resources(self):
         # Simulate extension of L3 attribute map
-        # First apply attribute extensions
-        for key in l3.RESOURCE_ATTRIBUTE_MAP.keys():
-            l3.RESOURCE_ATTRIBUTE_MAP[key].update(
-                l3_ext_gw_mode.EXTENDED_ATTRIBUTES_2_0.get(key, {}))
-            l3.RESOURCE_ATTRIBUTE_MAP[key].update(
-                extraroute.EXTENDED_ATTRIBUTES_2_0.get(key, {}))
-        # Finally add l3 resources to the global attribute map
-        attributes.RESOURCE_ATTRIBUTE_MAP.update(
-            l3.RESOURCE_ATTRIBUTE_MAP)
-        attributes.RESOURCE_ATTRIBUTE_MAP.update(
-            addr_apidef.RESOURCE_ATTRIBUTE_MAP)
+        l3.L3().update_attributes_map(
+            l3_egm_apidef.RESOURCE_ATTRIBUTE_MAP)
+        l3.L3().update_attributes_map(
+            xroute_apidef.RESOURCE_ATTRIBUTE_MAP)
         return (l3.L3.get_resources() +
                 address_scope.Address_scope.get_resources())
 
@@ -663,34 +1037,23 @@ class TestL3ExtensionManager(object):
         return []
 
 
-def backup_l3_attribute_map():
-    """Return a backup of the original l3 attribute map."""
-    return dict((res, attrs.copy()) for
-                (res, attrs) in six.iteritems(l3.RESOURCE_ATTRIBUTE_MAP))
-
-
-def restore_l3_attribute_map(map_to_restore):
-    """Ensure changes made by fake ext mgrs are reverted."""
-    l3.RESOURCE_ATTRIBUTE_MAP = map_to_restore
-
-
 class L3NatTest(test_l3_plugin.L3BaseForIntTests, NsxV3PluginTestCaseMixin,
                 test_address_scope.AddressScopeTestCase):
 
-    def _restore_l3_attribute_map(self):
-        l3.RESOURCE_ATTRIBUTE_MAP = self._l3_attribute_map_bk
-
     def setUp(self, plugin=PLUGIN_NAME, ext_mgr=None,
               service_plugins=None):
-        self._l3_attribute_map_bk = backup_l3_attribute_map()
         cfg.CONF.set_override('api_extensions_path', vmware.NSXEXT_PATH)
         cfg.CONF.set_default('max_routes', 3)
-        self.addCleanup(restore_l3_attribute_map, self._l3_attribute_map_bk)
         ext_mgr = ext_mgr or TestL3ExtensionManager()
         mock_nsx_version = mock.patch.object(nsx_plugin.utils,
                                              'is_nsx_version_2_0_0',
                                              new=lambda v: True)
         mock_nsx_version.start()
+        # Make sure the LB callback is not called on router deletion
+        self.lb_mock = mock.patch(
+            "vmware_nsx.services.lbaas.nsx_v3.lb_driver_v2."
+            "EdgeLoadbalancerDriverV2._check_lb_service_on_router")
+        self.lb_mock.start()
 
         super(L3NatTest, self).setUp(
             plugin=plugin, ext_mgr=ext_mgr, service_plugins=service_plugins)
@@ -699,6 +1062,7 @@ class L3NatTest(test_l3_plugin.L3BaseForIntTests, NsxV3PluginTestCaseMixin,
             self.plugin_instance.__module__,
             self.plugin_instance.__class__.__name__)
         self._plugin_class = self.plugin_instance.__class__
+        self.plugin_instance.fwaas_callbacks = None
 
     def test_floatingip_create_different_fixed_ip_same_port(self):
         self.skipTest('Multiple fixed ips on a port are not supported')
@@ -736,7 +1100,7 @@ class TestL3NatTestCase(L3NatTest,
         net_type = utils.NetworkTypes.L3_EXT
         expected = [('subnets', []), ('name', name), ('admin_state_up', True),
                     ('status', 'ACTIVE'), ('shared', False),
-                    (external_net.EXTERNAL, True),
+                    (extnet_apidef.EXTERNAL, True),
                     (pnet.NETWORK_TYPE, net_type),
                     (pnet.PHYSICAL_NETWORK, physical_network)]
         with self._create_l3_ext_network(physical_network) as net:
@@ -775,6 +1139,7 @@ class TestL3NatTestCase(L3NatTest,
         self.skipTest('not supported')
 
     def test_router_delete_with_lb_service(self):
+        self.lb_mock.stop()
         # Create the LB object - here the delete callback is registered
         lb_driver = lb_driver_v2.EdgeLoadbalancerDriverV2()
         with self.router() as router:
@@ -786,6 +1151,7 @@ class TestL3NatTestCase(L3NatTest,
                                   router['router']['id'])
         # Unregister callback
         lb_driver._unsubscribe_router_delete_callback()
+        self.lb_mock.start()
 
     def test_multiple_subnets_on_different_routers(self):
         with self.network() as network:
@@ -885,6 +1251,14 @@ class TestL3NatTestCase(L3NatTest,
                                   r['router']['id'],
                                   {'router': {'routes':
                                               routes}})
+
+                updates = {'admin_state_up': False}
+                self.assertRaises(n_exc.InvalidInput,
+                                  self.plugin_instance.update_router,
+                                  context.get_admin_context(),
+                                  r['router']['id'],
+                                  {'router': updates})
+
                 self._remove_external_gateway_from_router(
                     r['router']['id'],
                     s['subnet']['network_id'])
@@ -1320,6 +1694,79 @@ class TestL3NatTestCase(L3NatTest,
                 add_nat.assert_not_called()
                 delete_nat.assert_not_called()
 
+    def test_router_admin_state(self):
+        """It is not allowed to set the router admin-state to down"""
+        with self.router() as r:
+            self._update('routers', r['router']['id'],
+                         {'router': {'admin_state_up': False}},
+                         expected_code=exc.HTTPBadRequest.code)
+
+    def test_router_dhcp_relay_dhcp_enabled(self):
+        """Verify that the relay service is added to the router interface"""
+        self._enable_dhcp_relay()
+        with self.network() as network:
+            with mock.patch.object(self.plugin,
+                                  'validate_router_dhcp_relay'),\
+                self.subnet(network=network, enable_dhcp=True) as s1,\
+                self.router() as r1,\
+                mock.patch.object(self.plugin.nsxlib.logical_router_port,
+                                  'update') as mock_update_port:
+                self._router_interface_action('add', r1['router']['id'],
+                                              s1['subnet']['id'], None)
+                mock_update_port.assert_called_once_with(
+                    mock.ANY,
+                    relay_service_uuid=NSX_DHCP_RELAY_SRV,
+                    subnets=mock.ANY)
+
+    def test_router_dhcp_relay_dhcp_disabled(self):
+        """Verify that the relay service is not added to the router interface
+
+        If the subnet do not have enabled dhcp
+        """
+        self._enable_dhcp_relay()
+        with self.network() as network:
+            with mock.patch.object(self.plugin,
+                                  'validate_router_dhcp_relay'),\
+                self.subnet(network=network, enable_dhcp=False) as s1,\
+                self.router() as r1,\
+                mock.patch.object(self.plugin.nsxlib.logical_router_port,
+                                  'update') as mock_update_port:
+                self._router_interface_action('add', r1['router']['id'],
+                                              s1['subnet']['id'], None)
+                mock_update_port.assert_called_once_with(
+                    mock.ANY,
+                    relay_service_uuid=None,
+                    subnets=mock.ANY)
+
+    def test_router_dhcp_relay_no_ipam(self):
+        """Verify that a router cannot be created with relay and no ipam"""
+        # Add the relay service to the config and availability zones
+        self._enable_dhcp_relay()
+        self.assertRaises(n_exc.InvalidInput,
+                          self.plugin_instance.create_router,
+                          context.get_admin_context(),
+                          {'router': {'name': 'rtr'}})
+
+    def test_router_add_gateway_no_subnet_forbidden(self):
+        with self.router() as r:
+            with self.network() as n:
+                self._set_net_external(n['network']['id'])
+                self._add_external_gateway_to_router(
+                    r['router']['id'], n['network']['id'],
+                    expected_code=exc.HTTPBadRequest.code)
+
+    def test_router_add_gateway_no_subnet(self):
+        self.skipTest('No support for no subnet gateway set')
+
+    @mock.patch.object(nsx_plugin.NsxV3Plugin, 'validate_availability_zones')
+    def test_create_router_with_availability_zone(self, mock_validate_az):
+        name = 'rtr-with-zone'
+        zone = ['zone1']
+        mock_validate_az.return_value = None
+        with self.router(name=name, availability_zone_hints=zone) as rtr:
+            az_hints = rtr['router']['availability_zone_hints']
+            self.assertListEqual(zone, az_hints)
+
     def _test_route_update_illegal(self, destination):
         routes = [{'destination': destination, 'nexthop': '10.0.1.3'}]
         with self.router() as r:
