@@ -17,7 +17,7 @@ import contextlib
 
 import mock
 
-from neutron.extensions import external_net
+from neutron_lib.api.definitions import external_net as extnet_apidef
 from neutron_lib import context
 from neutron_lib.plugins import directory
 from neutron_vpnaas.db.vpn import vpn_models  # noqa
@@ -242,11 +242,11 @@ class TestVpnaasDriver(test_plugin.NsxVPluginV2TestCase):
     def test_create_vpn_service_legal(self):
         """Create a legal vpn service"""
         # create an external network with a subnet, and an exclusive router
-        providernet_args = {external_net.EXTERNAL: True}
+        providernet_args = {extnet_apidef.EXTERNAL: True}
         with self.network(name='ext-net',
                           providernet_args=providernet_args,
-                          arg_list=(external_net.EXTERNAL, )) as ext_net,\
-            self.subnet(ext_net, enable_dhcp=False),\
+                          arg_list=(extnet_apidef.EXTERNAL, )) as ext_net,\
+            self.subnet(ext_net),\
             self.router(router_type='exclusive',
                         external_gateway_info={'network_id':
                             ext_net['network']['id']}) as router,\
@@ -267,11 +267,11 @@ class TestVpnaasDriver(test_plugin.NsxVPluginV2TestCase):
     def test_create_vpn_service_on_shared_router(self):
         """Creating a service with shared router is not allowed"""
         # create an external network with a subnet, and a shared router
-        providernet_args = {external_net.EXTERNAL: True}
+        providernet_args = {extnet_apidef.EXTERNAL: True}
         with self.network(name='ext-net',
                           providernet_args=providernet_args,
-                          arg_list=(external_net.EXTERNAL, )) as ext_net,\
-            self.subnet(ext_net, enable_dhcp=False),\
+                          arg_list=(extnet_apidef.EXTERNAL, )) as ext_net,\
+            self.subnet(ext_net),\
             self.router(router_type='shared',
                         external_gateway_info={'network_id':
                             ext_net['network']['id']}) as router,\
@@ -292,11 +292,11 @@ class TestVpnaasDriver(test_plugin.NsxVPluginV2TestCase):
     def test_create_vpn_service_on_router_without_if(self):
         """Creating a service with unattached subnet is not allowed"""
         # create an external network with a subnet, and an exclusive router
-        providernet_args = {external_net.EXTERNAL: True}
+        providernet_args = {extnet_apidef.EXTERNAL: True}
         with self.network(name='ext-net',
                           providernet_args=providernet_args,
-                          arg_list=(external_net.EXTERNAL, )) as ext_net,\
-            self.subnet(ext_net, enable_dhcp=False),\
+                          arg_list=(extnet_apidef.EXTERNAL, )) as ext_net,\
+            self.subnet(ext_net),\
             self.router(router_type='exclusive',
                         external_gateway_info={'network_id':
                             ext_net['network']['id']}) as router,\
@@ -312,11 +312,11 @@ class TestVpnaasDriver(test_plugin.NsxVPluginV2TestCase):
     def test_create_vpn_service_without_subnet(self):
         """Creating a service without a subnet is not allowed"""
         # create an external network with a subnet, and an exclusive router
-        providernet_args = {external_net.EXTERNAL: True}
+        providernet_args = {extnet_apidef.EXTERNAL: True}
         with self.network(name='ext-net',
                           providernet_args=providernet_args,
-                          arg_list=(external_net.EXTERNAL, )) as ext_net,\
-            self.subnet(ext_net, enable_dhcp=False),\
+                          arg_list=(extnet_apidef.EXTERNAL, )) as ext_net,\
+            self.subnet(ext_net),\
             self.router(router_type='exclusive',
                         external_gateway_info={'network_id':
                             ext_net['network']['id']}) as router,\
